sdpvar v a t
sdpvar myeps0 myeps1
sdpvar myeps2 myeps3

sf_low=-2.05; 
sf_upp=-1.95;

Isp=2500;

lambda1=.5;   
lambda2=1;

tol0=1e-3;
tol1=1e-4;
tol2=1e-4;
tol3=1e-4;

%% a complete polynomial of degree n with m variables has C(m+n,n) monomials
[inv, c0] = polynomial([v a t], 6); 

[s1, c1] = polynomial([v a t], 4);
[s2, c2] = polynomial([v a t], 4);
[s3, c3] = polynomial([v a t], 4);
[s4, c4] = polynomial([v a t], 4);

flow=[a-1.622; (1/Isp)*a^2; 1]; 
dinv=[jacobian(inv,v), jacobian(inv,a), jacobian(inv,t)];
lie=dinv*flow;

%%------------------------------------------------------------------------------

%% initial set
p_init1=replace(inv,v,-2);
p_init2=replace(p_init1,a,1.622);
p_init=replace(p_init2,t,0);

f0=p_init <= -myeps0; %% tolerance to 0 by  tolerance 


%% transversality: convex condition 2
%% t>=0 /\ -t>=-0.128 ==> -lie-inv >= 0

f1=sos(-lie-lambda1*inv - s1*t*(-t+0.128) - myeps1);

%% safety
%% t>=0 /\ -t>=-0.128 /\ (v>sf_upp \/ v<sf_low) ==> inv > 0
f2=sos(inv - s2*t*(-t+0.128) - s3*(v-sf_upp)*(v-sf_low) - myeps2); 

%% transition: convex condition
%% t=0.128 ==> inv(v,a',t')<=inv(v,a,t)

rst_inv1 = replace(inv, a, -0.01*(a-1.622)-0.6*(v+2)+1.622);
rst_inv = replace(rst_inv1, t , 0);

f3=sos(lambda2*inv - rst_inv - s4*(t-0.128) - myeps3); %% s4 non-sos


%% ---------------------------------------------------------------------------------
FeasibilityConstraints=[f0,f1,f2,f3,sos(s1),sos(s2),sos(s3),myeps0>=tol0,myeps1>=tol1,myeps2>=tol2,myeps3>=tol3];

options=sdpsettings('verbose',1,'solver','sdpt3');    %% default option
%options=sdpsettings('verbose',1,'solver','sedumi' );
%options=sdpsettings('verbose',1,'solver','sedumi','sedumi.eps',1e-9);
%options=sdpsettings('verbose',1,'solver','sdpt3','sdpt3.gaptol',1.0000e-009);

solvesos(FeasibilityConstraints,[ ],options,[c0;c1;c2;c3;c4;myeps0;myeps1;myeps2;myeps3])

%% invariant
mono_inv=monolist([v a t], 6);
pinv=double(c0')*mono_inv;
sdisplay(clean(pinv,1e-3))


%ans = 
%
%    yalmiptime: 1.8230
%    solvertime: 0.8220
%          info: 'Successfully solved (SDPT3-4)'
%       problem: 0
%
% 3.628190622+2.3402*v-0.4797*a+0.4626*t+0.2286*v^2+0.5894*v*a+0.8191*a^2+0.0655*a*t+0.2738*t^2-0.0199*v^3+0.2025*v^2*a+0.3002*v*a^2-0.0836*a^3-0.2527*v^2*t-0.7062*v*a*t-1.0990*a^2*t-0.0803*v*t^2-0.2092*a*t^2-0.1730*t^3+0.0805*v^4+0.0860*v^3*a+0.0610*v^2*a^2-0.0173*v*a^3+0.0247*a^4+0.3202*v^3*t+0.3040*v^2*a*t-0.3542*v*a^2*t-0.0509*a^3*t+0.2341*v^2*t^2+0.1864*v*a*t^2+0.7879*a^2*t^2-0.0913*v*t^3+0.0892*a*t^3+0.1147*t^4-0.1097*v^4*t-0.3474*v^3*a*t-0.2516*v^2*a^2*t-0.0467*v*a^3*t-0.0748*a^4*t+0.2550*v^3*t^2-0.6129*v^2*a*t^2-0.1816*v*a^2*t^2-0.0906*a^3*t^2+0.0117*v^2*t^3-0.2252*v*a*t^3-0.1368*a^2*t^3+0.3376*v*t^4-0.2790*a*t^4-0.5651*t^5+0.1846*v^4*t^2+0.0785*v^3*t^3+0.3284*v^2*t^4+0.1384*v*t^5+0.4737*t^6
 
